# Nombres binaires fractionnaires

De la même façon qu’en base 10, il est possible de manipuler des nombres
fractionnaires en base 2. Ce sont les nombres qui peuvent s’écrire sous
la forme d’une fraction binaire, c’est-à-dire une fraction dont le
numérateur est un entier relatif, et le dénominateur est une puissance
de 2. C'est un sous-ensemble de l'ensemble des nombres rationnels.
Comme dans le cas de la base 10, on peut également représenter ces
nombres en utilisant une virgule pour séparer la partie entière de la
partie fractionnaire. 

1.  Donner sous forme de fraction binaire le nombre décimal 0,5., puis donner
    sa représentation en notation positionnelle en base 2.

2.  Donner sous forme de fraction binaire le nombre décimal 2,25., puis donner
    sa représentation en notation positionnelle en base 2.

3.  Donner sous forme de fraction binaire le nombre décimal 42,75., puis donner
    sa représentation en notation positionnelle en base 2.

4.  Peut-on représenter le nombre décimal 0,1 en base 2 ?

5.  Soit le nombre qui s'écrit 101,1001 en base 2. Donner sa représentation
    sous forme de fraction binaire, de fraction décimale, puis en notation
    positionnelle en base 10.


# Le format simple précision - codage

1.  Rappeler les différents champs qui composent le format IEEE754 simple
    précision. Pour chacun, donner sa taille et sa position.

2.  Rappeler la formule permettant de calculer la valeur codée par un nombre en
    écriture normalisée.

3.  Coder le nombre 10.

4.  Coder le nombre 3,375.

5.  Coder le nombre  −7,78125.

6.  Coder le nombre 0,1. Quelle est la précision du codage ?

7.  Donner la valeur du plus grand et du plus petit nombre (en valeur absolue)
    qu’il est possible de coder en écriture normalisée.

8.  Donner un nombre entier appartenant à l’intervalle délimité par les nombres
    obtenues en réponse à la question précédente, et qu’il n’est pas possible
    de coder.

# Le format simple précision - décodage

1.  Décoder le nombre 0x414a8000

2.  Décoder le nombre 0x42280000

3.  Décoder le nombre 0xc316c000


# Nombres en écriture dénormalisée

1.  Rappeler le codage d’un nombre en écriture dénormalisée.

2.  Rappeler la formule permettant de calculer la valeur d'un nombre en
    écriture dénormalisée. Quelles sont les deux différences que l’on peut
    identifier par rapport à l’écriture normalisée ?

3.  Donner la valeur du plus grand et du plus petit (en valeur absolue) nombre
    différent de zéro qu’il est possible de coder en écriture dénormalisée.

4.  Quel est l’intérêt de l’écriture dénormalisée ?


# Autres valeurs

En plus des nombres en écriture normalisée et dénormalisée, trois
valeurs exceptionnelles peuvent être codées.

-   Pour chacune, rappeler son nom, son codage, et expliquer son
    utilité.

-   De telles valeurs n’existent pas pour l’arithmétique entière telle qu’elle
    est implantée dans les machines actuelles. Dès lors, que se passe-t-il
    quand on exécute une opération d’arithmétique entière dont le résultat
    n’est pas défini (par exemple une division par zéro) ?


# Programme d'analyse du codage

Récupérer sur le dépôt git le fichier `fshow.go`

Identifier ce que font les fonctions `getX1`, `getX2`, `getX3`, puis les
renommer avec des noms plus adaptés. Ensuite, analyser et compléter le
code de la fonction `showFloat`. Une fois complété, voici les sorties
attendues :

    $ ./fshow 100.001

    Floating point value 100.0009995
    Bit Representation 0x42c80083, sign = 0, exponent = 0x85, fraction = 0x480083
    Normalized.  +1.5625156164 X 2^(6)

    $ ./fshow 0x4321CAFE

    Floating point value 161.7929382
    Bit Representation 0x4321cafe, sign = 0, exponent = 0x86, fraction = 0x21cafe
    Normalized.  +1.2640073299 X 2^(7)

    $ ./fshow 0x7F800000

    Floating point value +Inf
    Bit Representation 0x7f800000, sign = 0, exponent = 0xff, fraction = 0x000000
    +Infinity


# Précision relative en écriture normalisée

1.  Donner l’écriture normalisée de 1, 2 et 4.

2.  Combien de nombres différents de l’intervalle \[1, 2\[ est-il
    possible de coder ? quelle est la distance entre ces nombres
    (donner la réponse en base 2 et l’ordre de grandeur en base 10) ?

3.  Même questions mais en considérant l’intervalle \[2, 4\[.

4.  Même questions mais en considérant l’intervalle
    \[2<sup>127</sup>, 2<sup>128</sup>\[.

5.  Que peut-on déduire concernant la précision du codage IEEE754 ?


# Cancellation dans les opérations d'arithmétique flottante

Récupérer le fichier `bigsmall.go`.
Lorsqu’il est exécuté, ce programme affiche les lignes suivantes :

```
bsb =  0
bbs =  1e-06
```

Expliquer.


# Critère d'arrêt des boucles et arithmétique flotttante 

Récupérer le fichier `sqrt.go`.
Ce programme  calcule la racine carré de `y` par la méthode de Newton. Lorsque
`y` vaut 3, le programme affiche les lignes suivantes :

```
2
5.15
1.75
2.8662622
1.7321428
1.9564607
1.7320508
1.744921
sqrt( 3 ) =  1.7320508
```

En revanche, lorsque `y` vaut 2, l'exécution entre dans une boucle infinie. Les
premières lignes de la sortie sont :

```
1.5
5.1
1.4166666
2.7460785
1.4142157
1.7371949
1.4142135
1.4442381
1.4142135
1.4145256
1.4142135
1.4142135
```

Expliquer pourquoi le programme boucle et proposer une correction qui
assure que le programme se termine quelle que soit la valeur de `y`.
Programmer et tester cette correction.

